#ifndef _CHRONOPTICS_TOF_KEA_CAMERA_H_
#define _CHRONOPTICS_TOF_KEA_CAMERA_H_

#include <chronoptics/tof/camera.h>
#include <chronoptics/tof/processing_config.h>
#include <chronoptics/tof/camera_config.h>

#ifdef __cplusplus
extern "C" {
#endif

/** The main interface to the kea camera
 */
typedef struct tof_kea_camera* tof_kea_camera_t;

typedef struct tof_kea_camera const* tof_kea_camera_ct;

/** Destruct tof_kea_camera */
TOF_EXPORT void tof_kea_camera_delete(tof_kea_camera_t ptr);

/** Construct the kea camera with the given processing and camera config.
 * @param ptr Pointer to class
 * @param serial The serial number of the camera to connect to, if the camera is
 * available over both usb and gige it will be connected over usb. If an empty
 * string is supplied the camera will connect to the first usb camera, if no usb
 * camera is connected it will connect to the first gige camera that replies
 * (this reply time is random, so you might connect to a different camera each
 * time if you've got multiple cameras connected to your network). If no camera
 * is found with the given serial number a nullptr and error is returned.
 * @param processing_config The processing config
 * @param camera_config The camera config
 * @param error Pointer to error
 * @return New object
 */
TOF_EXPORT tof_kea_camera_t tof_kea_camera_new_both_configs(const char* serial, tof_processing_config_ct processing_config, tof_camera_config_ct camera_config, tof_error_t *error);

/** Construct the kea camera
 * @param ptr Pointer to class
 * @param serial The serial number of the camera to connect to, if the camera is
 * available over both usb and gige it will be connected over usb. If an empty
 * string is supplied the camera will connect to the first usb camera, if no usb
 * camera is connected it will connect to the first gige camera that replies
 * (this reply time is random, so you might connect to a different camera each
 * time if you've got multiple cameras connected to your network). If no camera
 * is found with the given serial number a nullptr and error is returned.
 * @param processing_config The processing config
 * @param error Pointer to error
 * @return New object
 */
TOF_EXPORT tof_kea_camera_t tof_kea_camera_new(const char* serial, tof_processing_config_ct processing_config, tof_error_t *error);

/** Construct the kea camera with a processing config generated by
 * default_processing from the camera config
 * @param ptr Pointer to class
 * @param serial The serial number of the camera to connect to, if the camera is
 * available over both usb and gige it will be connected over usb. If an empty
 * string is supplied the camera will connect to the first usb camera, if no usb
 * camera is connected it will connect to the first gige camera that replies
 * (this reply time is random, so you might connect to a different camera each
 * time if you've got multiple cameras connected to your network). If no camera
 * is found with the given serial number a nullptr and error is returned.
 * @param error Pointer to error
 * @return New object
 */
TOF_EXPORT tof_kea_camera_t tof_kea_camera_new_simple(const char* serial, tof_error_t *error);

/** Check whether this kea camera can do on camera processing
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return On camera processing capable
 */
TOF_EXPORT bool tof_kea_camera_on_camera_processing_capable(tof_kea_camera_ct ptr, tof_error_t *error);

/** Get on camera processing
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return True on camera processing, false on host processing
 */
TOF_EXPORT bool tof_kea_camera_get_on_camera_processing(tof_kea_camera_ct ptr, tof_error_t *error);

/** Set on camera processing
 * @param ptr Pointer to class
 * @param on_camera_processing True on camera processing, false on host
 * processing
 * @param error Pointer to error
 */
TOF_EXPORT void tof_kea_camera_set_on_camera_processing(tof_kea_camera_t ptr, bool on_camera_processing, tof_error_t *error);

/** Set the camera config
 * @param ptr Pointer to class
 * @param camera_config Camera config
 * @param error Pointer to error
 */
TOF_EXPORT void tof_kea_camera_set_camera_config(tof_kea_camera_t ptr, tof_camera_config_ct camera_config, tof_error_t *error);

/** Set both the camera and processing config
 * @param ptr Pointer to class
 * @param camera_config Camera config
 * @param processing_config Processing config
 * @param error Pointer to error
 */
TOF_EXPORT void tof_kea_camera_set_configurations(tof_kea_camera_t ptr, tof_camera_config_ct camera_config, tof_processing_config_ct processing_config, tof_error_t *error);

/** Check whether camera supports multiple configurations
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Multiple configurations possible
 */
TOF_EXPORT bool tof_kea_camera_multiple_configurations_possible(tof_kea_camera_ct ptr, tof_error_t *error);

/** Set multiple configurations
 * @param ptr Pointer to class
 * @param camera_configs Camera configurations
 * @param camera_configs_size The size of the above array
 * @param pro_configs Processing configurations
 * @param pro_configs_size The size of the above array
 * @param error Pointer to error
 */
TOF_EXPORT void tof_kea_camera_set_multiple_configurations(tof_kea_camera_t ptr, const tof_camera_config_t* camera_configs, size_t camera_configs_size, const tof_processing_config_t* pro_configs, size_t pro_configs_size, tof_error_t *error);

/** the processing config of the camera
 * @param ptr Pointer to class
 * @param config The processing config
 * @param error Pointer to error
 */
TOF_EXPORT void tof_kea_camera_set_process_config(tof_kea_camera_t ptr, tof_processing_config_ct config, tof_error_t *error);

/** Simple class containing information about found cameras
 */
typedef struct tof_discovered_camera* tof_discovered_camera_t;

typedef struct tof_discovered_camera const* tof_discovered_camera_ct;

/** Destruct tof_discovered_camera */
TOF_EXPORT void tof_discovered_camera_delete(tof_discovered_camera_t ptr);

/** The serial
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Serial
 */
TOF_EXPORT const char* tof_discovered_camera_serial(tof_discovered_camera_ct ptr, tof_error_t *error);

/** Additional information about the discovered camera
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Additional information
 */
TOF_EXPORT const char* tof_discovered_camera_info(tof_discovered_camera_ct ptr, tof_error_t *error);

/** Get the camera model name
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Camera model
 */
TOF_EXPORT const char* tof_discovered_camera_model(tof_discovered_camera_ct ptr, tof_error_t *error);

/** Check whether this camera is a TUI camera
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Is tui
 */
TOF_EXPORT bool tof_discovered_camera_is_tui(tof_discovered_camera_ct ptr, tof_error_t *error);

/** Discover all available cameras
 * @param available_cameras All available cameras
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_discover_cameras(tof_discovered_camera_t* available_cameras, size_t capacity, tof_error_t *error);

/** Construct a kea camera using the gige interface. This could be useful in two
 * cases. The first is when the camera is connected with both usb and ethernet
 * but you want to use ethernet. The second is when for some reason the default
 * maximum packet size is too large.
 * @param processing_config The processing config
 * @param serial The serial number of the camera to connect to, if the camera is
 * available over both usb and gige it will be connected over usb. If an empty
 * string is supplied the camera will connect to the first usb camera, if no usb
 * camera is connected it will connect to the first gige camera that replies
 * (this reply time is random, so you might connect to a different camera each
 * time if you've got multiple cameras connected to your network). If no camera
 * is found with the given serial number a nullptr and error is returned.
 * @param packet_size The packet size to use for communication over ethernet
 * @param error Pointer to error
 * @return A kea camera
 */
TOF_EXPORT tof_kea_camera_t tof_create_kea_camera_gige(tof_processing_config_ct processing_config, const char* serial, uint16_t packet_size, tof_error_t *error);

#ifdef __cplusplus
}
#endif

#endif
