import ctypes
from .__library__ import lib, get_error
from .camera_config import *
from .camera import *
from .processing_config import *


class ConfigMode:
    """
    Optimized configurations for different use cases
    """

    LONG_RANGE_DYNAMIC_SCENE = 1
    "Suitable for most applications"

    MEDIUM_RANGE_DYNAMIC_SCENE = 2
    "For fast moving applications"

    MEDIUM_RANGE_BALANCED = 3
    "Balanced between range, speed and accuracy"

    SHORT_RANGE_HIGH_SPEED = 4
    "Maximum speed mode for short range and fast moving applications"

    LONG_RANGE_HIGH_RESOLUTION = 5
    "Full resolution long range"

    MEDIUM_RANGE_STATIC_SCENE = 6
    "Full resolution for slow changing scene"

    FAR_OUT_STATIC_SCENE = 7
    "Full resolution for slow changing scene"



def configModeCameraConfig(camera, config_mode):
    """
    Get the camera config associated with the config mode

    Parameters
    ----------
    camera : Camera
        Camera
    config_mode : ConfigMode
        Config mode

    Returns
    ----------
    CameraConfig
        Camera Config
    """
    return CameraConfig(ptr = lib.tof_config_mode_camera_config(camera._ptr, config_mode, get_error()))


def configModeProcessingConfig(camera, config_mode):
    """
    Get the processing config associated with the config mode

    Parameters
    ----------
    camera : Camera
        Camera
    config_mode : ConfigMode
        Config mode

    Returns
    ----------
    ProcessingConfig
        Processing Config
    """
    return ProcessingConfig(ptr = lib.tof_config_mode_processing_config(camera._ptr, config_mode, get_error()))


def hdrPossible(config):
    """
    Check whether hdr is possible with current camera config

    Parameters
    ----------
    config : CameraConfig
        The camera config to check

    Returns
    ----------
    bool
        Whether hdr is possible
    """
    return lib.tof_hdr_possible(config._ptr, get_error())


def phaseUnwrapPossible(config):
    """
    Check whether phase unwrap is possible with current camera config

    Parameters
    ----------
    config : CameraConfig
        The camera config to check

    Returns
    ----------
    bool
        Whether phase unwrap is possible
    """
    return lib.tof_phase_unwrap_possible(config._ptr, get_error())


def applyDefaultProcessingConfig(camera):
    """
    Generate processing config based on camera configuration and calibration.

    Parameters
    ----------
    camera : Camera
        A camera
    """
    return lib.tof_apply_default_processing_config(camera._ptr, get_error())


